/************************************************
/* Siggen serial programming routine header
/* for PIC16F684
/* by Christopher Durkin (cjdurkin@gmail.com)
/***********************************************/
#ifndef __SIGGEN_H
#define __SIGGEN_H

#include <pic.h>

#define SCLK RC0
#define SDATA RC1
#define SYNC RC2
#define LE RC3
#define OUT1 RC4
#define OUT2 RC5
#define BIT7 0b10000000

#define PLLROWS 4
#define PLLCOLS 3

#define bitset(var,bitno) ((var) |= (1 << (bitno)))
#define bitclr(var,bitno) ((var) &= ~(1 << (bitno)))

#define int_enable() (GIE = 1)
#define int_disable() (GIE = 0)

typedef unsigned char byte;

byte portcvar = 0xFF;

void write_portc(byte towrite) {
	portcvar = towrite;
	PORTC = towrite;
}

void pinset(byte bitnumber) {
	byte scratch;
    scratch = portcvar;
    bitset(scratch,bitnumber);
	write_portc(scratch);
	scratch = ~scratch; //wasting time
}
void pinclr(byte bitnumber) {
	byte scratch;
    scratch = portcvar;
    bitclr(scratch,bitnumber);
	write_portc(scratch);
	scratch = ~scratch; //wasting time
}

#define clockset() pinset(0)
#define clockclr() pinclr(0)
#define dataset() pinset(1)
#define dataclr() pinclr(1)
#define muxen() pinclr(2)
#define muxdis() pinset(2)
#define pllen() pinclr(3)
#define plldis() pinset(3)

// set pll for 2.44 GHz
const static byte pll_bytes[PLLROWS][PLLCOLS]= {
  {
    0x5F,
    0x80,
    0xA6
  },
  {
    0x00,
    0x00,
    0x28
  },
  {
    0x00,
    0x98,
    0x21
  },
  {
    0x5F,
    0x80,
    0xA2
  }
};


void program_pll(void) {

  byte j,k,m,bangthis;

  clockclr();
  
  for( j=0; j < PLLROWS; j++) {
	pllen();
    for( m=0; m < PLLCOLS; m++) {
      bangthis = pll_bytes[j][m];
      for( k=0; k<8; k++ ) {                                                    
        if ((bangthis & BIT7) != 0) {
			dataset();
		} else {
			dataclr();
		}
        clockset();
        clockclr();
        bangthis = bangthis << 1;
      } // for k
    } // for m
	plldis();
  } // for j

} // program_pll

void program_mux(char channel_select) {
  byte k,current_byte;
  clockset(); // bring up clock, data is falling edge
  muxen(); //mux enable
  current_byte = channel_select;
  for(k=0;k<8;k++) {                                                    
    if ((current_byte & BIT7) != 0) {
		dataset();
	} else {
		dataclr();
	}                                   
    clockclr(); // clock down
    current_byte = current_byte << 1;
    clockset(); // clock up
  }
  muxdis(); //mux disable
}

void pic_init(void) {
  GIE = 0; //disable global interrupts for now

  /* OUTPUT INITIALIZATION */
  TRISA = 0xFF; // all in
  PORTA = 0x00; // all low (don't matter though)
  TRISC = 0x00; // all out
  write_portc(0xFF); // all high (does matter!)

  /* OTHER STUFF */
  CMCON0 = 0x07;
  TMR0 = 0; // start tmr0 at 0
  T0IF = 0; //timer 0 int flag
  T0IE = 1; //timer 0 interrupt
  OPTION = 0x0F; // prescaler to WDT, 1:128
  ANSEL = 0x00;

  /* VARIOUS UNUSED */
  //VRCON = 0; 
  //ADCON0 = 0x01;
  //ADIE = 0;
  //ADIF = 0;
  //GPIF = 0;
}

#endif
