/*
Little utility routines for running the 
Microchip(tm) PICkit(tm) FLASH Starter Kit
USB device programmer.

Orion Sky Lawlor, olawlor@acm.org, 2003/8/4 (public domain)
*/
#ifndef __OSL_PIC_PICKIT_H
#define __OSL_PIC_PICKIT_H

#include <pic.h>

typedef unsigned char byte;

// Light this LED, numbered from 0..7
void led(byte led)             
{
	/* These tables come from the wiring on the 
	   PICkit's LEDs.  We tristate all but two pins,
	   and either forward or reverse bias the two 
	   non-tristated pins.  This drives exactly one
	   LED on the display.
	 */
	const static byte trsTable[8]={
		0b11001111,
		0b11001111,
		0b11101011,
		0b11101011,
		0b11011011,
		0b11011011,
		0b11111001,
		0b11111001
	};
	const static byte gpTable[8]={
		0b00010000,
		0b00100000,
		0b00010000,
		0b00000100,
		0b00100000,
		0b00000100,
		0b00000100,
		0b00000010
	};
	TRISA = trsTable[led];
	PORTA = gpTable[led];
}

void bangbits(void) {
	const static byte tobebanged[12]={
          0x5F,
          0x80,
          0xA6,
          0x00,
          0x00,
          0x28,
          0x00,
          0x98,
          0x21,
          0x5F,
          0x80,
          0xA2
	};

        byte k,m,current_byte;

        // mux select
        RC2 = 1; //mux enable
        current_byte = 0x01;
        for(k=0;k<8;k++) {                                                    
          RC1 = current_byte && 0b10000000;                                   
          RC0 = 0; // clock down                                              
          current_byte = current_byte << 1;                                   
          RC0 = 1; // clock up                                                
        }
        RC2 = 0; //mux disable


        for(m=0;m<12;m++) {
          current_byte = tobebanged[m];
          for(k=0;k<8;k++) {
            RC1 = current_byte && 0b10000000;
            RC0 = 0; // clock down
            current_byte = current_byte << 1;
            RC0 = 1; // clock up
          }
        }
}


// Busywait for this long (FIXME: should use sleep & timer)
void busywait(byte ms) {
	for (;ms!=0;ms--) 
	{ // Wait for one millisecond (depends on compiler, optimizer)
          byte c;
          CLRWDT();
          for (c=250;c!=0;c--) {}
	}
}

// Turn all hardware off (e.g., at startup)
void clear(void) {
	TRISA = 0xFF;
	PORTA = 0x00; 
	//VRCON = 0; 
	CMCON0 = 0x07;
	//TMR0 = 0;
	//OPTION = 0x80;
	ANSEL = 0x00;
	//ADCON0 = 0x01;
	//T0IE = 0;
	//ADIE = 0;
	//GIE = 0;
	//T0IF = 0;
	//ADIF = 0;
	//GPIF = 0;
}

#endif
